;************************************************************************************
;*																					*
;*							320 x 240 Pixel LCD Controller			        		*
;*							  mit RTC DS1307 und TWI-Bus							*
;*	Aufbau jeweils einer Pixelreihe (40 Bytes) im SRAM und bertragung zum Display	*
;*																					*
;************************************************************************************
;
;       			 ATMega16 
;   				+--+-+--+                        
;	    (XCK/TO)PB0	|  |_|	|	PA0(ADC0)                       
;   	    (T1)PB1	|		|	PA1(ADC1)                   
;    (INT2/AIN0)PB2	|		|	PA2(ADC2)              
;	  (OC0/AIN1)PB3 |       |   PA3(ADC3)	
;	        (SS)PB4 |       |   PA4(ADC4) 	
;         (MOSI)PB5 |       |   PA5(ADC5) 	
;		  (MISO)PB6 |		|	PA6(ADC6)	
;		   (SCK)PB7 |		|	PA7(ADC7)
;			  RESET |		|	AREF
;	    		VCC |       |   GND
;   			GND |       |   AVcc
;  			  XTAL2 |		|	PC7(TOSC2)            
;  			  XTAL1 |		|	PC6(TOSC1)          
;		   (RXD)PD0	|		|	PC5(TDI)               
;		   (TXD)PD1	|		|	PC4(TDO)                     
;		  (INT0)PD2	|		|	PC3(TMS)                      
;	      (INT1)PD3 |		|	PC2(TCK)                 
;		  (OC1B)PD4 |		|	PC1(SDA)
;		  (OC1A)PD5 |		|	PC0(SCL)
;	      (ICP1)PD6 |		|	PD7(OC2)
;   				+-------+                           
;
; PA0	O	LCD_D0
; PA1	O	LCD_D1
; PA2	O	LCD_D2 
; PA3	O	LCD_D3
; PA4	O	nc
; PA5	O	nc
; PA6	O	nc
; PA7	O	nc   
; PB0	I	nc(pullup)	
; PB1	I	nc(pullup)	
; PB2	I	nc(pullup)	 
; PB3	I	nc(pullup)	
; PB4	I	nc(pullup)	
; PB5	I	MOSI	
; PB6	I	MISO
; PB7	I	SCK   
; PC0 	I	SCL 
; PC1 	I	SDA 
; PC2	I	nc(pullup) 
; PC3	I	nc(pullup) 
; PC4	I	nc(pullup)
; PC5	I	nc(pullup)
; PC6	I	Stelltaster(pullup)
; PC7	I	Schalter Sek./Min.(pullup)
; PD0	O	PCLK
; PD1	O	Load
; PD2	O	Frame
; PD3	O	AC
; PD4	O	ONOFF  
; PD5	O	Kontrast
; PD6	O	nc 
; PD7	O	nc

.include "m16def.inc"

;Register:
.def regsave		=r7
.def clockL			=r8
.def clockH			=r9
.def X_Cnt			=r10

.def temp			=r16
.def Y_Cnt			=r17
.def C_Cnt			=r18

.def tempdat		=r19				;I2C data transfer register
.def tmp			=r20
.def t_tmp			=r21
.def RTC_Flag		=r22				;Flag fr Auslesen der RTC

.def Wert_akt		=r23				;aktueller Wert von der RTC und I2C address and direction register
.def X_lang			=r24				;Lnge des Datenblocks und Zeichen (RTC)
.def X_off			=r25				;X Offset des Datenblocks
;zustzlich: X, Y, Z

;***************************************************************************************
;LCD Konstanten
;PORTD
.equ 	CLOCK		=0					;Pixelclock
.equ 	LP			=1					;Latch Puls (HSync)
.equ 	FRAME		=2					;First Line Marker (VSync)
.equ 	ONOFF		=4					;LCD (VLCD) Enable
.equ 	PWM			=5					;PWM fr Contrast

;Werte
.equ Framerate		=50					;Refreshrate
.equ F_CPU			=16000000			;CPU Freq
.equ RX_Buffer		=32

; Interne Konstanten
.equ HFREQ			=Framerate*240
.equ TDIV			=((F_CPU/8+(HFREQ)/2)/HFREQ)
.equ CSize			=24					;Zeilenhhe 24 Pixel
.equ XSize			=40					;Zeilenbreite 40 Byte
.equ YSize			=240/CSize			;Anzahl Zeilen


;************************************************************************************
;Timerkonstanten

;Vorgaben fr die RTC (!!!!!Uhr stellen!!!!!!!)
;-------------------------------------------------------------------------
.equ Sekunden		=0x30				;Sekunden=30
.equ Minuten		=0x39				;Minuten=39
.equ Stunden		=0x11				;Stunden=10 (24-Hour Mode)
.equ Wochentag		=0x02				;Wochentag (1=Montag,2=Dienstag...)
.equ Datum			=0x08				;Tag
.equ Monat			=0x06				;Mrz
.equ Jahr			=0x10				;(20)10
;-------------------------------------------------------------------------

.equ Jahr_			=192				;Lnge der einzelnen Datenblcke
.equ Dat_			=58
.equ Zeit_			=432
.equ Sek_			=58

;TWI - Konstanten
.equ TWIrd				=1
.equ TWIwr				=0
.equ TWIDispAdr			=0b11010000		;Timer Adresse

.equ TWIPORT			=PORTC

.equ C_Start			=0x00			;Start Clock
.equ Sek_Register		=0x00
.equ Min_Register		=0x01
.equ Std_Register		=0x02
.equ Tag_Register		=0x03
.equ Dat_Register		=0x04
.equ Mon_Register		=0x05
.equ Jahr_Register		=0x06
.equ Con_Register		=0x07

;************************************************************************************************
;Konstante fr Schalter und Taster
.equ TS_PIN				=PINC
.equ T_PIN				=7
.equ S_PIN				=6

;************************************************************************************************
.MACRO offset							;Makro zur Berechnung des Null-offsets mit aktuellem Wert
	clr		r0
	clr		r1
	mul		ZL, Wert_akt
	movw	r3:r2, r1:r0
	mul		ZH, Wert_akt
	add 	r2, r1
	adc		r3, r0
	mul		temp, C_Cnt
	add		r2, r0
	adc		r3, r1
.ENDM

.dseg
Inhalt:	.byte	40						;eine Pixelreile, 40 byte Lnge und 24 Pixel Hhe

;umgerechnete (BCD-binr) Werte werden hier gespeichert:
Sek_Z:		.byte 1
Sek_E:		.byte 1
Min_Z:		.byte 1
Min_E:		.byte 1
Std_Z:		.byte 1
Std_E:		.byte 1
WTag_dummy: .byte 1
WTag_akt:	.byte 1
Dat_Z:		.byte 1
Dat_E:		.byte 1
Mon_dummy:	.byte 1
Mon_akt:	.byte 1
Jahr_Z:		.byte 1
Jahr_E:		.byte 1						;Hunderter und Tausender mit Software
										
;	Interrupt Vektoren
;****************************************************************************
.cseg
.org 0
	rjmp Reset

.org OC2addr
	rjmp LCD_Line
	
;****************************************************************************
;	CPU Register Initialisierung
;****************************************************************************
Reset:
	ldi		temp, low(RAMEND)
	out		SPL, temp					;setup stack pointer
	ldi		temp, high(RAMEND)
	out		SPH, temp					;setup stack pointer

;LCD einrichten und starten
	ser 	temp						;set Register -> 0xFF
	out 	DDRD, temp					;PCLK, LOAD, FLM, M, ONOFF
	out 	DDRA, temp					;D0 - D3

;Timer 1 (16 bit) zur Kontrastregelung
	ldi		temp, (1<<WGM10)|(1<<COM1A1)|(1<<COM1A0); 8bit PWM, Ausgabe auf OC1A
	out		TCCR1A, temp				;Timer Counter Control Register 1A (Timer 1)
	
	ldi 	temp, 1						;Systemtakt/256 -> 62,5kHz PWM
	out 	TCCR1B, temp				;Timer Counter Control Register 1B (Timer 1)

	ldi 	temp, 128					;low byte des 16 bit Vergleichswertes
	out		OCR1AL, temp				;Output Compare Register Timer 1A Low

;Timer 2 (8 bit) zur Interruptsteuerung fr LCD
	ldi 	temp, (1<<WGM21)|2			;CTC (Clear Timer on Compare match), Vorteiler = 8
	out 	TCCR2, temp

	ldi 	temp, TDIV-1
	out 	OCR2, temp					;Output Compare Register Timer 2

	ldi 	temp, (1<<OCIE2)			;Compare2 Interrupt: Beginn der Datenausgabe
	out 	TIMSK, temp

;TWI init
	ldi		temp,0b11111100				;pullup einschalten
	out		TWIPORT,temp			

	ldi		temp, 18							
	out		TWBR, temp					;Bitrate fr Bustakt 100kHz
	ldi		temp, 0	
	out		TWSR, temp					;Vorteiler TWPS = 1
	
	;rcall	RTC_Neustart				;ggf. Neustart der Uhr
	;rcall	RTC_Write					;Stellen der Uhr auf die oben vorgegebenen Werte
	rcall	RTC_Read					;Auslesen der Uhr und Speicherung im SRAM

;****************************************************************************
;	Variablen Initialisierung
;****************************************************************************
	ldi		YL, low(Inhalt)				;Zeiger zum bertragen ins SRAM und ins Display
	ldi		YH, high(Inhalt)
	clr		C_Cnt
	inc		RTC_Flag					;da RTC Startwerte geladen sofort LCD Ausgabe

	sei									;LCD Interrupt starten
	sbi 	PORTD, ONOFF

;****************************************************************************
;	Hauptprogramm
;****************************************************************************
Mainloop:
	sbis	TS_PIN, T_PIN
	rjmp	T_Abfrage
	tst		RTC_Flag					;eine Aktion bereits erledigt?
	breq	Lesen
	rjmp 	Mainloop

;Lesen der RTC im BCD Format,Umwandlung in binre Zehner und Einer
;und Speicherung im SRAM,
;Das Auslesen erfolgt einmal nach jedem Bildaufbau.
Lesen:									;eine Aktion bereits erledigt?
	cpi		RTC_Flag, 1					;wennja, zurck zu Mainloop
	breq	Mainloop
	ldi		RTC_Flag, 1	
	rcall	RTC_Read
	rjmp	Mainloop

;Stellen der Uhr. S_PIN = Auswahl Sekunden oder Minuten
;T_PIN = Rckstellung der Sekunden auf Null, bzw. Minuten erhhen.
T_Abfrage:
	ldi		tempdat, 8

T_Ab:
	ldi		t_tmp, 0xFF
	dec		tempdat
	brne	Wait_loop	
	sbic	TS_PIN, T_PIN
	rjmp	Mainloop
	sbis	TS_PIN, S_PIN
	rjmp	Sek_Write
	rjmp	Min_Write	

Wait_loop:
	ldi		tmp, 0xFF

Wait_loop1:
	dec     tmp
	brne	Wait_loop1
	dec		t_tmp
	brne	Wait_loop
	rjmp	T_Ab

;**********************************************************************
;LCD Daten ausgeben
LCD_Line:								;Interrupt Einsprung
	ldi 	temp, XSize
	mov 	X_Cnt, temp
	in 		temp, PORTD
	cbr 	temp, 1<<CLOCK
	mov 	clockL, temp
	sbr 	temp, 1<<CLOCK
	mov 	clockH, temp

hloop:									;Loop fr 1 Zeile
	out 	PORTD, clockL				;PCLK L			
	ld		temp, Y+
	nop
	out 	PORTD, clockH				;PCLK H			
	out 	PORTA, temp					;Low Nibble ausgeben
	out 	PORTD, clockL				;PCLK L			
	swap 	temp										
	out 	PORTA, temp					;High Nibble ausgeben
	out 	PORTD, clockH				;PCLK H			
	dec 	X_Cnt										
	brne 	hloop										
										
	out 	PORTD, clockL				;PCLK L

	sbi 	PORTD, LP
	inc 	C_Cnt						;Zeilencounter erhhen
	nop
	nop
	cbi 	PORTD, LP
	cbi 	PORTD, FRAME
	ldi		YL, low(Inhalt)				;Zeiger zurcksetzen
	ldi		YH, high(Inhalt)

	rjmp	skipcloop	

skipcloop:								
	cpi 	C_Cnt, CSize				;Zeilenende (24 Pixelreihen)?
	brne	Z_Wahl						;Zeile nicht komplett, dann exit		
	clr		C_Cnt						;neue Zeile einrichten
	inc 	Y_Cnt						;Zeilencounter erhhen
	cpi 	Y_Cnt, YSize				;Bildende ?
	brne 	Z_Wahl

	sbi 	PORTD, FRAME
	clr		Y_Cnt
	clr		RTC_Flag

Z_Wahl:									;Sprung in die aktuelle Zeile
	ldi		ZL,LOW(Sprungtabelle)
	ldi		ZH,HIGH(Sprungtabelle)
	clr		temp
	add		ZL, Y_Cnt
	adc		ZH, temp
	ijmp

exit_int:
	out 	sreg, regsave
	reti

Sprungtabelle:
	rjmp	exit_int
	rjmp	Zeile_2
	rjmp	Zeile_3
	rjmp	Zeile_4
	rjmp	Zeile_5
	rjmp	Zeile_6
	rjmp	Zeile_7
	rjmp	Zeile_8
	rjmp	Zeile_9
	rjmp	Zeile_10

.INCLUDE "RTC_Start_Lesen.asm"

.INCLUDE "RTC_Stellen.asm"

.INCLUDE "Zeilenaufbau.asm"
;*******************************************************
;*** CG ROM Daten ***

.INCLUDE "Daten_Wochentage.asm"

.INCLUDE "Daten_Zahlen_mittelgross.asm"																		

.INCLUDE "Daten_Zahlen_klein.asm"																		

.INCLUDE "Daten_Monate.asm"

.INCLUDE "Daten_Zahlen_gross.asm"																		





